#!/usr/bin/env node
var path = require('path');
var arg = require('arg');
var fs = require('fs/promises');
var require$$0 = require('tty');
var bunchee = require('bunchee');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var path__default = /*#__PURE__*/_interopDefault(path);
var arg__default = /*#__PURE__*/_interopDefault(arg);
var fs__default = /*#__PURE__*/_interopDefault(fs);
var require$$0__default = /*#__PURE__*/_interopDefault(require$$0);

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

var picocolors = {exports: {}};

let tty = require$$0__default.default;
let isColorSupported = !("NO_COLOR" in process.env || process.argv.includes("--no-color")) && ("FORCE_COLOR" in process.env || process.argv.includes("--color") || process.platform === "win32" || tty.isatty(1) && process.env.TERM !== "dumb" || "CI" in process.env);
let formatter = (open, close, replace = open)=>(input)=>{
        let string = "" + input;
        let index = string.indexOf(close, open.length);
        return ~index ? open + replaceClose(string, close, replace, index) + close : open + string + close;
    };
let replaceClose = (string, close, replace, index)=>{
    let start = string.substring(0, index) + replace;
    let end = string.substring(index + close.length);
    let nextIndex = end.indexOf(close);
    return ~nextIndex ? start + replaceClose(end, close, replace, nextIndex) : start + end;
};
let createColors = (enabled = isColorSupported)=>({
        isColorSupported: enabled,
        reset: enabled ? (s)=>`\x1b[0m${s}\x1b[0m` : String,
        bold: enabled ? formatter("\x1b[1m", "\x1b[22m", "\x1b[22m\x1b[1m") : String,
        dim: enabled ? formatter("\x1b[2m", "\x1b[22m", "\x1b[22m\x1b[2m") : String,
        italic: enabled ? formatter("\x1b[3m", "\x1b[23m") : String,
        underline: enabled ? formatter("\x1b[4m", "\x1b[24m") : String,
        inverse: enabled ? formatter("\x1b[7m", "\x1b[27m") : String,
        hidden: enabled ? formatter("\x1b[8m", "\x1b[28m") : String,
        strikethrough: enabled ? formatter("\x1b[9m", "\x1b[29m") : String,
        black: enabled ? formatter("\x1b[30m", "\x1b[39m") : String,
        red: enabled ? formatter("\x1b[31m", "\x1b[39m") : String,
        green: enabled ? formatter("\x1b[32m", "\x1b[39m") : String,
        yellow: enabled ? formatter("\x1b[33m", "\x1b[39m") : String,
        blue: enabled ? formatter("\x1b[34m", "\x1b[39m") : String,
        magenta: enabled ? formatter("\x1b[35m", "\x1b[39m") : String,
        cyan: enabled ? formatter("\x1b[36m", "\x1b[39m") : String,
        white: enabled ? formatter("\x1b[37m", "\x1b[39m") : String,
        gray: enabled ? formatter("\x1b[90m", "\x1b[39m") : String,
        bgBlack: enabled ? formatter("\x1b[40m", "\x1b[49m") : String,
        bgRed: enabled ? formatter("\x1b[41m", "\x1b[49m") : String,
        bgGreen: enabled ? formatter("\x1b[42m", "\x1b[49m") : String,
        bgYellow: enabled ? formatter("\x1b[43m", "\x1b[49m") : String,
        bgBlue: enabled ? formatter("\x1b[44m", "\x1b[49m") : String,
        bgMagenta: enabled ? formatter("\x1b[45m", "\x1b[49m") : String,
        bgCyan: enabled ? formatter("\x1b[46m", "\x1b[49m") : String,
        bgWhite: enabled ? formatter("\x1b[47m", "\x1b[49m") : String
    });
picocolors.exports = createColors();
picocolors.exports.createColors = createColors;
var picocolorsExports = picocolors.exports;
var pc = /*@__PURE__*/ getDefaultExportFromCjs(picocolorsExports);

const logger = {
    log (...arg) {
        console.log('  ', ...arg);
    },
    warn (...arg) {
        console.warn(' ⚠️', ...arg);
    },
    error (...arg) {
        console.error(' ⨯', ...arg);
    },
    info (...arg) {
        console.log(' ✓', ...arg);
    }
};
function paint(prefix, prefixColor, ...arg) {
    if (pc.isColorSupported) {
        console.log(' ' + pc[prefixColor](prefix), ...arg);
    } else {
        console.log(' ' + prefix, ...arg);
    }
}

function exit(err) {
    logger.error(err);
    process.exit(1);
}
async function hasPackageJson(cwd) {
    return await fileExists(path__default.default.resolve(cwd, 'package.json'));
}
async function getPackageMeta(cwd) {
    const pkgFilePath = path__default.default.resolve(cwd, 'package.json');
    let targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(await fs__default.default.readFile(pkgFilePath, {
            encoding: 'utf-8'
        }));
    } catch (_) {}
    return targetPackageJson;
}
async function fileExists(filePath) {
    try {
        await fs__default.default.access(filePath);
        return true;
    } catch (err) {
        if (err.code === 'ENOENT') {
            return false;
        }
        throw err;
    }
}

var version = "4.2.4";

const helpMessage = `
Usage: bunchee [options]

Options:
  -v, --version          output the version number
  -w, --watch            watch src files changes
  -m, --minify           compress output. default: false
  -o, --output <file>    specify output filename
  -f, --format <format>  type of output (esm, amd, cjs, iife, umd, system), default: esm
  -h, --help             output usage information
  --external <mod>       specify an external dependency, separate by comma
  --no-external          do not bundle external dependencies
  --target <target>      js features target: swc target es versions. default: es2015
  --runtime <runtime>    build runtime (nodejs, browser). default: browser
  --env <env>            inlined process env variables, separate by comma. default: NODE_ENV
  --cwd <cwd>            specify current working directory
  --sourcemap            enable sourcemap generation, default: false
  --dts                  determine if need to generate types, default: false
`;
function help() {
    logger.log(helpMessage);
}
async function lintPackage(cwd) {
    // Not package.json detected, skip package linting
    if (!await hasPackageJson(cwd)) {
        return;
    }
    const { publint } = await import('publint');
    const { formatMessage } = await import('publint/utils');
    const { messages } = await publint({
        pkgDir: cwd,
        level: 'error'
    });
    const pkg = await getPackageMeta(cwd);
    for (const message of messages){
        console.log(formatMessage(message, pkg));
    }
}
function parseCliArgs(argv) {
    let args;
    args = arg__default.default({
        '--cwd': String,
        '--dts': Boolean,
        '--output': String,
        '--format': String,
        '--watch': Boolean,
        '--minify': Boolean,
        '--help': Boolean,
        '--version': Boolean,
        '--runtime': String,
        '--target': String,
        '--sourcemap': Boolean,
        '--env': String,
        '--external': String,
        '--no-external': Boolean,
        '-h': '--help',
        '-v': '--version',
        '-w': '--watch',
        '-o': '--output',
        '-f': '--format',
        '-m': '--minify'
    }, {
        permissive: true,
        argv
    });
    const source = args._[0];
    const parsedArgs = {
        source,
        format: args['--format'],
        file: args['--output'],
        watch: args['--watch'],
        minify: args['--minify'],
        sourcemap: !!args['--sourcemap'],
        cwd: args['--cwd'],
        dts: args['--dts'],
        help: args['--help'],
        version: args['--version'],
        runtime: args['--runtime'],
        target: args['--target'],
        external: !!args['--no-external'] ? null : args['--external'],
        env: args['--env']
    };
    return parsedArgs;
}
async function run(args) {
    var _args_external;
    const { source, format, watch, minify, sourcemap, target, runtime, dts, env } = args;
    const cwd = args.cwd || process.cwd();
    const file = args.file ? path__default.default.resolve(cwd, args.file) : undefined;
    const bundleConfig = {
        dts,
        file,
        format,
        cwd,
        target,
        runtime,
        external: args.external === null ? null : ((_args_external = args.external) == null ? void 0 : _args_external.split(',')) || [],
        watch: !!watch,
        minify: !!minify,
        sourcemap: sourcemap === false ? false : true,
        env: (env == null ? void 0 : env.split(',')) || []
    };
    if (args.version) {
        return logger.log(version);
    }
    if (args.help) {
        return help();
    }
    const entry = source ? path__default.default.resolve(cwd, source) : '';
    try {
        await bunchee.bundle(entry, bundleConfig);
    } catch (err) {
        if (err.name === 'NOT_EXISTED') {
            help();
            return exit(err);
        }
        throw err;
    }
    // watching mode
    if (watch) {
        logger.log(`Watching assets in ${cwd}...`);
        return;
    }
    // build mode
    console.log();
    paint('✓', 'green', `bunchee ${version} build completed`);
    await lintPackage(cwd);
}
async function main() {
    let params, error;
    try {
        params = parseCliArgs(process.argv.slice(2));
    } catch (err) {
        error = err;
    }
    if (error || !params) {
        if (!error) help();
        return exit(error);
    }
    await run(params);
}
main().catch(exit);
